import os
import shutil
import subprocess
import pytest


def pytest_addoption(parser):
	parser.addini('sample_medium', 'size of meidum nzb (megabytes)', default=192)
	parser.addini('sample_large', 'size of large nzb (megabytes)', default=1024)


@pytest.fixture(scope='session', autouse=True)
def prepare_testdata(request):
	print('Preparing test data for "download"')
	pytest.check_config()

	nserv_datadir = pytest.config.getini('nserv_datadir')
	nzbget_bin = pytest.config.getini('nzbget_bin')
	sevenzip_bin = pytest.config.getini('sevenzip_bin')

	if not os.path.exists(nserv_datadir):
		print('Creating nserv datadir')
		os.makedirs(nserv_datadir)

	if not os.path.exists(nserv_datadir + '/medium.nzb'):
		sizemb = int(pytest.config.getini('sample_medium'))
		create_test_file(nserv_datadir + '/medium', sevenzip_bin, sizemb, 50)

	if not os.path.exists(nserv_datadir + '/large.nzb'):
		sizemb = int(pytest.config.getini('sample_large'))
		create_test_file(nserv_datadir + '/large', sevenzip_bin, sizemb, 50)

	if not os.path.exists(nserv_datadir + '/medium.nzb') or not os.path.exists(nserv_datadir + '/large.nzb'):
		if 0 != subprocess.call([nzbget_bin, '--nserv', '-d', nserv_datadir, '-v', '2', '-z', '500000', '-q']):
			pytest.exit('Test file generation failed')

	nzbget_srcdir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(__file__))))
	if not os.path.exists(nserv_datadir + '/small.nzb'):
		if not os.path.exists(nserv_datadir + '/small'):
			os.makedirs(nserv_datadir + '/small')
		shutil.copyfile(nzbget_srcdir +'/COPYING', nserv_datadir + '/small/small.dat')

	if not os.path.exists(nserv_datadir + '/small-obfuscated.nzb'):
		if not os.path.exists(nserv_datadir + '/small-obfuscated'):
			os.makedirs(nserv_datadir + '/small-obfuscated')
		shutil.copyfile(nzbget_srcdir +'/COPYING', nserv_datadir + '/small-obfuscated/fsdkhKHGuwuMNBKskd')

	if 0 != subprocess.call([nzbget_bin, '--nserv', '-d', nserv_datadir, '-v', '2', '-z', '3000', '-q']):
		pytest.exit('Test file generation failed')


def create_test_file(bigdir, sevenzip_bin, sizemb, partmb):
	print('Preparing test file (' + str(sizemb) + 'MB)')

	if not os.path.exists(bigdir):
		os.makedirs(bigdir)

	f = open(bigdir + '/' + str(sizemb) + 'mb.dat', 'wb')
	for n in xrange(sizemb // partmb):
		print('Writing block %i from %i' % (n + 1, sizemb // partmb))
		f.write(os.urandom(partmb * 1024 * 1024))
	f.close()

	if 0 != subprocess.call([sevenzip_bin, 'a', bigdir + '/' + str(sizemb) + 'mb.7z', '-mx=0', '-v' + str(partmb) + 'm', bigdir + '/' + str(sizemb) + 'mb.dat']):
		pytest.exit('Test file generation failed')

	os.remove(bigdir + '/' + str(sizemb) + 'mb.dat')



nzbget_options = ['ParCheck=force', 'ParQuick=yes', 'PostStrategy=sequential']

def test_parchecker_healthy(nserv, nzbget):
	hist = nzbget.download_nzb('parchecker2.nzb')
	assert hist['Status'] == 'SUCCESS/PAR'

def test_parchecker_repair(nserv, nzbget):
	nzb_content = nzbget.load_nzb('parchecker2.nzb')
	nzb_content = nzb_content.replace('parchecker2/testfile.7z.001?16=45000:3000', 'parchecker2/testfile.7z.001?16=45000:3000!0')
	hist = nzbget.download_nzb('parchecker.repair.nzb', nzb_content)
	assert hist['Status'] == 'SUCCESS/PAR'

def test_parchecker_middle(nserv, nzbget):
	nzb_content = nzbget.load_nzb('parchecker2.nzb')
	nzb_content = nzb_content.replace('parchecker2/testfile.7z.001?16=45000:3000', 'parchecker2/testfile.7z.001?16=45000:3000!0')
	hist = nzbget.download_nzb('parchecker.middle.nzb', nzb_content)
	assert hist['Status'] == 'SUCCESS/PAR'

def test_parchecker_last(nserv, nzbget):
	nzb_content = nzbget.load_nzb('parchecker2.nzb')
	nzb_content = nzb_content.replace('<segment bytes="3000" number="18">parchecker2/testfile.7z.001?18=51000:200</segment>', '')
	hist = nzbget.download_nzb('parchecker.last.nzb', nzb_content)
	assert hist['Status'] == 'SUCCESS/PAR'



nzbget_options = ['ParCheck=auto', 'ParQuick=no', 'PostStrategy=rocket']

def test_parchecker_healthy(nserv, nzbget):
	hist = nzbget.download_nzb('parchecker2.nzb')
	assert hist['Status'] == 'SUCCESS/HEALTH'

def test_parchecker_repair(nserv, nzbget):
	nzbget.api.pausepost();
	nzb_content = nzbget.load_nzb('parchecker2.nzb')
	nzb_content = nzb_content.replace('parchecker2/testfile.7z.001?16=45000:3000', 'parchecker2/testfile.7z.001?16=45000:3000!0')
	nzbget.append_nzb('parchecker.1.nzb', nzb_content, dupemode='FORCE')
	nzbget.append_nzb('parchecker.2.nzb', nzb_content, dupemode='FORCE')
	nzbget.append_nzb('parchecker.3.nzb', nzb_content, dupemode='FORCE')

	while True:
		status = nzbget.api.status()
		if status['RemainingSizeMB'] == 0:
			break
		time.sleep(0.1)

	nzbget.api.resumepost();
	hist1 = nzbget.wait_nzb('parchecker.3.nzb');
	hist2 = nzbget.wait_nzb('parchecker.3.nzb');
	hist3 = nzbget.wait_nzb('parchecker.3.nzb');
	assert hist1['Status'] == 'SUCCESS/PAR'
	assert hist2['Status'] == 'SUCCESS/PAR'
	assert hist3['Status'] == 'SUCCESS/PAR'



nzbget_options = ['ParRename=no', 'RarRename=no', 'ParCheck=manual', 'DirectRename=no']

def test_rename_obf1(nserv, nzbget):
	hist = nzbget.download_nzb('obfuscated1.nzb', unpack=True)
	assert hist['Status'] == 'WARNING/DAMAGED'

def test_rename_obf1ch(nserv, nzbget):
	nzb_content = nzbget.load_nzb('obfuscated1.nzb')
	nzb_content = nzb_content.replace(';5mb.7z', ';abc')
	nzb_content = nzb_content.replace(';parrename.par2', ';def')
	nzb_content = nzb_content.replace('.par2&', '&')
	hist = nzbget.download_nzb('obfuscated1-changed.nzb', nzb_content, unpack=True)
	assert hist['Status'] == 'WARNING/DAMAGED'

def test_rename_obf2(nserv, nzbget):
	hist = nzbget.download_nzb('obfuscated2.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/HEALTH'

def test_parjoin_1(nserv, nzbget):
	hist = nzbget.download_nzb('parjoin1.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_parjoin_2(nserv, nzbget):
	hist = nzbget.download_nzb('parjoin2.nzb', unpack=True)
	assert hist['Status'] == 'WARNING/DAMAGED'

def test_parjoin_3(nserv, nzbget):
	hist = nzbget.download_nzb('parjoin3.nzb', unpack=True)
	assert hist['Status'] == 'WARNING/DAMAGED'



import os
import shutil
import subprocess
import pytest


@pytest.fixture(scope='session', autouse=True)
def prepare_testdata(request):
	print('Preparing test data for "parcheck"')

	nserv_datadir = pytest.config.getini('nserv_datadir')
	nzbget_bin = pytest.config.getini('nzbget_bin')

	if not os.path.exists(nserv_datadir):
		print('Creating nserv datadir')
		os.makedirs(nserv_datadir)

	nzbget_srcdir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(__file__))))

	testdata_dir = nzbget_srcdir + '/tests/testdata'
	if not os.path.exists(nserv_datadir + '/parchecker'):
		shutil.copytree(testdata_dir +'/parchecker', nserv_datadir + '/parchecker')
	if not os.path.exists(nserv_datadir + '/parchecker2'):
		shutil.copytree(testdata_dir +'/parchecker2', nserv_datadir + '/parchecker2')

	if 0 != subprocess.call([nzbget_bin, '--nserv', '-d', nserv_datadir, '-v', '2', '-z', '3000', '-q']):
		pytest.exit('Test file generation failed')



nzbget_options = ['ParRename=yes', 'RarRename=yes', 'ParCheck=auto', 'DirectRename=yes', 'ParScan=full']

def test_parjoin_1(nserv, nzbget):
	hist = nzbget.download_nzb('parjoin1.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/PAR'

def test_parjoin_2(nserv, nzbget):
	hist = nzbget.download_nzb('parjoin2.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/PAR'

def test_parjoin_3(nserv, nzbget):
	hist = nzbget.download_nzb('parjoin3.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/PAR'



nzbget_options = ['ParRename=yes', 'RarRename=yes', 'UnpackIgnoreExt=.cbr', 'DirectRename=no']

def test_parrename(nserv, nzbget):
	hist = nzbget.download_nzb('parrename.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_parrename_backup(nserv, nzbget):
	nzb_content = nzbget.load_nzb('parrename.nzb')
	nzb_content = nzb_content.replace('parrename/parrename.par2?', 'parrename/parrename.par2.damaged?')
	hist = nzbget.download_nzb('parrename.backup.nzb', nzb_content, unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'
	log = nzbget.api.loadlog(int(hist['ID']), 0, 1000)
	renamed = False
	for entry in log:
		print(entry['Text'])
		if entry['Text'].find('Successfully renamed') > -1 and entry['Text'].find('archive') == -1:
			renamed = True
	assert renamed == True


def test_rarrename_rar3(nserv, nzbget):
	hist = nzbget.download_nzb('rarrename3.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_rarrename_rar5(nserv, nzbget):
	hist = nzbget.download_nzb('rarrename5.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_rarrename_rar3oldnam(nserv, nzbget):
	hist = nzbget.download_nzb('rarrename3oldnam.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_rarrename_rar3badext(nserv, nzbget):
	hist = nzbget.download_nzb('rarrename3badext.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_rarrename_rar5badext(nserv, nzbget):
	hist = nzbget.download_nzb('rarrename5badext.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_rarrename_2sets(nserv, nzbget):
	hist = nzbget.download_nzb('rarrename2sets.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_rarrename_rar3damaged(nserv, nzbget):
	nzb_content = nzbget.load_nzb('rarrename3sm.nzb')
	nzb_content = nzb_content.replace('abc.32?14=6500:500', 'abc.32?14=6500:500!2')
	hist = nzbget.download_nzb('rarrename3sm.nzb', nzb_content, unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_rarrename_rar3encdata(nserv, nzbget):
	hist = nzbget.download_nzb('rarrename3encdata.nzb', unpack=True, params=[('*unpack:password', '123')])
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_rarrename_rar5encdata(nserv, nzbget):
	hist = nzbget.download_nzb('rarrename5encdata.nzb', unpack=True, params=[('*unpack:password', '123')])
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_rarrename_rar3encnam(nserv, nzbget):
	hist = nzbget.download_nzb('rarrename3encnam.nzb', unpack=True, params=[('*unpack:password', '123')])
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_rarrename_rar5encnam(nserv, nzbget):
	hist = nzbget.download_nzb('rarrename5encnam.nzb', unpack=True, params=[('*unpack:password', '123')])
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_rarrename_rar3ignoreext(nserv, nzbget):
	hist = nzbget.download_nzb('rar3ignoreext.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/HEALTH'


def test_rename_obf1(nserv, nzbget):
	hist = nzbget.download_nzb('obfuscated1.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_rename_obf1ch(nserv, nzbget):
	nzb_content = nzbget.load_nzb('obfuscated1.nzb')
	nzb_content = nzb_content.replace(';5mb.7z', ';abc')
	nzb_content = nzb_content.replace(';parrename', ';def')
	nzb_content = nzb_content.replace('.par2&', '&')
	hist = nzbget.download_nzb('obfuscated1-changed.nzb', nzb_content, unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_rename_obf2(nserv, nzbget):
	hist = nzbget.download_nzb('obfuscated2.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_parjoin_1(nserv, nzbget):
	hist = nzbget.download_nzb('parjoin1.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/PAR'

def test_parjoin_2(nserv, nzbget):
	hist = nzbget.download_nzb('parjoin2.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/PAR'

def test_parjoin_3(nserv, nzbget):
	hist = nzbget.download_nzb('parjoin3.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/PAR'



import base64

nzbget_options = ['HealthCheck=none', 'DupeCheck=yes']

def test_dupecheck_small_copy(nserv, nzbget):
	hist = nzbget.download_nzb('small.nzb', dupemode = 'SCORE')
	assert hist['Status'] == 'SUCCESS/HEALTH'
	nzb_content = nzbget.load_nzb('small.nzb')
	hist = nzbget.download_nzb('small.copy.nzb', nzb_content, dupemode = 'SCORE')
	assert hist['Status'] == 'DELETED/COPY'

def test_dupecheck_small_id(nserv, nzbget):
	nzbget.clear()
	hist = nzbget.download_nzb('small.nzb', dupemode = 'SCORE')
	assert hist['Status'] == 'SUCCESS/HEALTH'
	nzb_content = nzbget.load_nzb('small.nzb')
	nzbcontent64 = base64.standard_b64encode(nzb_content)
	id = nzbget.api.append('small.copy2.nzb', nzbcontent64, 'test', 0, False, False, '', 0, 'SCORE', [])
	assert id > 0



import re

nzbget_options = ['ParRename=yes', 'RarRename=yes', 'ParCheck=auto', 'DirectRename=yes']

def test_rename_obf1(nserv, nzbget):
	hist = nzbget.download_nzb('obfuscated1.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_rename_obf1ch(nserv, nzbget):
	nzb_content = nzbget.load_nzb('obfuscated1.nzb')
	nzb_content = nzb_content.replace(';5mb.7z', ';abc')
	nzb_content = nzb_content.replace(';parrename', ';def')
	nzb_content = nzb_content.replace('.par2&', '&')
	hist = nzbget.download_nzb('obfuscated1-changed.nzb', nzb_content, unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_rename_obf2(nserv, nzbget):
	hist = nzbget.download_nzb('obfuscated2.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_rename_obf3(nserv, nzbget):
	hist = nzbget.download_nzb('obfuscated3.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_rename_obf1dm(nserv, nzbget):
	nzb_content = nzbget.load_nzb('obfuscated1.nzb')
	nzb_content = nzb_content.replace('abc.01?4=300000:100000', 'abc.01?4=300000:100000!0')
	hist = nzbget.download_nzb('obfuscated1-damaged.nzb', nzb_content, unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_rename_obf1dmf(nserv, nzbget):
	nzb_content = nzbget.load_nzb('obfuscated1.nzb')
	nzb_content = nzb_content.replace('abc.01?1=0:100000', 'abc.01?1=0:100000!0')
	hist = nzbget.download_nzb('obfuscated1-damaged-first.nzb', nzb_content, unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_rename_obf1dmf2(nserv, nzbget):
	nzb_content = nzbget.load_nzb('obfuscated1.nzb')
	nzb_content = nzb_content.replace('abc.01?1=0:100000', 'abc.01?1=0:100000!0')
	nzb_content = re.sub(r'abc.00\?1=0:\d+', r'\g<0>!0', nzb_content)
	hist = nzbget.download_nzb('obfuscated1-damaged-first2.nzb', nzb_content, unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_rename_obf1dmp(nserv, nzbget):
	nzb_content = nzbget.load_nzb('obfuscated1.nzb')
	nzb_content = re.sub(r'parrename.vol0+1.par2?1=0:\d+', r'\g<0>!0', nzb_content)
	hist = nzbget.download_nzb('obfuscated1-damaged-par.nzb', nzb_content, unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_rename_obf3dm(nserv, nzbget):
	nzb_content = nzbget.load_nzb('obfuscated3.nzb')
	nzb_content = nzb_content.replace('abc.01?17=1600000:100000', 'abc.01?17=1600000:100000!0')
	hist = nzbget.download_nzb('obfuscated3-damaged.nzb', nzb_content, unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_rename_obf3dmf(nserv, nzbget):
	nzb_content = nzbget.load_nzb('obfuscated3.nzb')
	nzb_content = nzb_content.replace('abc.01?11=0:100000', 'abc.01?11=0:100000!0')
	hist = nzbget.download_nzb('obfuscated3-damaged-first.nzb', nzb_content, unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_rename_obf3dmf2(nserv, nzbget):
	nzb_content = nzbget.load_nzb('obfuscated3.nzb')
	nzb_content = nzb_content.replace('abc.01?11=0:100000', 'abc.01?11=0:100000!0')
	nzb_content = re.sub(r'abc.00\?1=0:\d+', r'\g<0>!0', nzb_content)
	hist = nzbget.download_nzb('obfuscated3-damaged-first2.nzb', nzb_content, unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_parchecker_healthy(nserv, nzbget):
	hist = nzbget.download_nzb('parchecker2.nzb')
	assert hist['Status'] == 'SUCCESS/HEALTH'

def test_parchecker_repair(nserv, nzbget):
	nzb_content = nzbget.load_nzb('parchecker2.nzb')
	nzb_content = nzb_content.replace('parchecker2/testfile.7z.001?16=45000:3000', 'parchecker2/testfile.7z.001?16=45000:3000!0')
	hist = nzbget.download_nzb('parchecker2.repair.nzb', nzb_content)
	assert hist['Status'] == 'SUCCESS/PAR'

def test_parchecker_dmp(nserv, nzbget):
	nzb_content = nzbget.load_nzb('parchecker2.nzb')
	nzb_content = nzb_content.replace('parchecker2/testfile.7z.par2?1=0:3000', 'parchecker2/testfile.7z.par2?1=0:3000!0')
	hist = nzbget.download_nzb('parchecker2.damagedpar.nzb', nzb_content)
	assert hist['Status'] == 'SUCCESS/HEALTH'
	for entry in nzbget.api.loadlog(hist['ID'], 0, 10000):
		assert entry['Kind'] != 'ERROR', entry['Text']

def test_parjoin_1(nserv, nzbget):
	hist = nzbget.download_nzb('parjoin1.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/PAR'

def test_parjoin_2(nserv, nzbget):
	hist = nzbget.download_nzb('parjoin2.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/PAR'

def test_parjoin_3(nserv, nzbget):
	hist = nzbget.download_nzb('parjoin3.nzb', unpack=True)
	assert hist['Status'] == 'SUCCESS/PAR'



nzbget_options = ['HealthCheck=park', 'ArticleCache=500', 'DirectWrite=yes']

def test_retry_small_redownload(nserv, nzbget):
	nzb_content = nzbget.load_nzb('small.nzb')
	nzb_content = nzb_content.replace('?3=6000:3000', '?3=6000:3000!2')
	hist = nzbget.download_nzb('small.bad2.nzb', nzb_content)
	assert hist['Status'] == 'FAILURE/HEALTH'
	nzbget.api.editserver(2, True)
	nzbget.api.editqueue('HistoryRedownload', 0, '', [hist['NZBID']])
	hist = nzbget.wait_nzb('small.bad2.nzb')
	nzbget.api.editserver(2, False)
	assert hist['Status'] == 'SUCCESS/HEALTH'

def test_retry_small_retryfailed(nserv, nzbget):
	nzb_content = nzbget.load_nzb('small.nzb')
	nzb_content = nzb_content.replace('?3=6000:3000', '?3=6000:3000!2')
	hist = nzbget.download_nzb('small.bad3.nzb', nzb_content)
	assert hist['Status'] == 'FAILURE/HEALTH'
	nzbget.api.editqueue('HistoryRetryFailed', 0, '', [hist['NZBID']])
	hist = nzbget.wait_nzb('small.bad3.nzb')
	assert hist['Status'] == 'FAILURE/HEALTH'
	nzbget.api.editserver(2, True)
	nzbget.api.editqueue('HistoryRetryFailed', 0, '', [hist['NZBID']])
	hist = nzbget.wait_nzb('small.bad3.nzb')
	nzbget.api.editserver(2, False)
	assert hist['Status'] == 'SUCCESS/HEALTH'

def test_retry_medium_failed(nserv, nzbget):
	nzb_content = nzbget.load_nzb('medium.nzb')
	nzb_content = nzb_content.replace('000000:500000', '000000:500000!2')
	hist = nzbget.download_nzb('medium.nzb', nzb_content)
	assert hist['Status'] == 'FAILURE/HEALTH'
	assert hist['DeleteStatus'] == 'HEALTH'

def test_retry_medium_redownload(nserv, nzbget):
	nzb_content = nzbget.load_nzb('medium.nzb')
	nzb_content = nzb_content.replace('0000000:500000', '0000000:500000!2')
	hist = nzbget.download_nzb('medium.bad.nzb', nzb_content)
	assert hist['Status'] == 'FAILURE/HEALTH'
	nzbget.api.editserver(2, True)
	nzbget.api.editqueue('HistoryRedownload', 0, '', [hist['NZBID']])
	hist = nzbget.wait_nzb('medium.bad.nzb')
	nzbget.api.editserver(2, False)
	assert hist['Status'] == 'SUCCESS/HEALTH'

def test_retry_medium_retryfailed(nserv, nzbget):
	nzb_content = nzbget.load_nzb('medium.nzb')
	nzb_content = nzb_content.replace('0000000:500000', '0000000:500000!2')
	hist = nzbget.download_nzb('medium.bad3.nzb', nzb_content)
	assert hist['Status'] == 'FAILURE/HEALTH'
	nzbget.api.editqueue('HistoryRetryFailed', 0, '', [hist['NZBID']])
	hist = nzbget.wait_nzb('medium.bad3.nzb')
	assert hist['Status'] == 'FAILURE/HEALTH'
	nzbget.api.editserver(2, True)
	nzbget.api.editqueue('HistoryRetryFailed', 0, '', [hist['NZBID']])
	hist = nzbget.wait_nzb('medium.bad3.nzb')
	nzbget.api.editserver(2, False)
	assert hist['Status'] == 'SUCCESS/HEALTH'



#!/usr/bin/env python
#
# Logger post-processing script for NZBGet
#
# Copyright (C) 2013-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


##############################################################################
### NZBGET POST-PROCESSING SCRIPT                                          ###

# Save nzb log into a file.
#
# This script saves the download and post-processing log of nzb-file
# into file _nzblog.txt in the destination directory.
#
# NOTE: This script requires Python to be installed on your system.

### NZBGET POST-PROCESSING SCRIPT                                          ###
##############################################################################


import os
import sys
import datetime

try:
	from xmlrpclib import ServerProxy # python 2
	from urllib2 import quote
except ImportError:
	from xmlrpc.client import ServerProxy # python 3
	from urllib.parse import quote

# Exit codes used by NZBGet
POSTPROCESS_SUCCESS=93
POSTPROCESS_NONE=95
POSTPROCESS_ERROR=94

# Check if the script is called from nzbget 15.0 or later
if not 'NZBOP_NZBLOG' in os.environ:
	print('*** NZBGet post-processing script ***')
	print('This script is supposed to be called from nzbget (15.0 or later).')
	sys.exit(POSTPROCESS_ERROR)

if not os.path.exists(os.environ['NZBPP_DIRECTORY']):
	print('Destination directory doesn\'t exist, exiting')
	sys.exit(POSTPROCESS_NONE)

# To get the item log we connect to NZBGet via XML-RPC and call
# method "loadlog", which returns the log for a given nzb item.
# For more info visit http://nzbget.net/RPC_API_reference

# First we need to know connection info: host, port and password of NZBGet server.
# NZBGet passes all configuration options to post-processing script as
# environment variables.
host = os.environ['NZBOP_CONTROLIP'];
port = os.environ['NZBOP_CONTROLPORT'];
username = os.environ['NZBOP_CONTROLUSERNAME'];
password = os.environ['NZBOP_CONTROLPASSWORD'];

if host == '0.0.0.0': host = '127.0.0.1'

# Build a URL for XML-RPC requests
rpcUrl = 'http://%s:%s@%s:%s/xmlrpc' % (quote(username), quote(password), host, port);

# Create remote server object
server = ServerProxy(rpcUrl)

# Call remote method 'loadlog'
nzbid = int(os.environ['NZBPP_NZBID'])
log = server.loadlog(nzbid, 0, 10000)

# Now iterate through entries and save them to the output file
if len(log) > 0:
	f = open('%s/_nzblog.txt' % os.environ['NZBPP_DIRECTORY'], 'wb')
	for entry in log:
		f.write((u'%s\t%s\t%s\n' % (entry['Kind'], datetime.datetime.fromtimestamp(int(entry['Time'])), entry['Text'])).encode('utf8'))
	f.close()

# All OK, returning exit status 'POSTPROCESS_SUCCESS' (int <93>) to let NZBGet know
# that our script has successfully completed.
sys.exit(POSTPROCESS_SUCCESS)



nzbget_options = ['HealthCheck=delete']

def test_medium_health(nserv, nzbget):
	nzb_content = nzbget.load_nzb('medium.nzb')
	nzb_content = nzb_content.replace('.7z', '-does-not-exist.7z')
	hist = nzbget.download_nzb('medium-health.nzb', nzb_content)
	assert hist['Status'] == 'FAILURE/HEALTH'
	assert hist['DeleteStatus'] == 'HEALTH'



nzbget_options = ['SaveQueue=no']

import os

nzbget = None

def prepare_queue(nzbget_instance):
	global nzbget

	if nzbget != None:
		nzbget.api.editqueue('GroupSort', 'Name', [])
		check_list([1,2,3,4,5,6,7,8,9])
		return

	print('prepare_queue')
	nzbget = nzbget_instance

	nzbget.api.pausedownload()

	nzbget_srcdir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(__file__))))
	fullfilename = nzbget_srcdir + '/tests/testdata/nzbfile/plain.nzb'
	in_file = open(fullfilename, 'r')
	nzb_content = in_file.read()
	in_file.close()

	nzbget.append_nzb('1', nzb_content)
	nzbget.append_nzb('2', nzb_content)
	nzbget.append_nzb('3', nzb_content)
	nzbget.append_nzb('4', nzb_content)
	nzbget.append_nzb('5', nzb_content)
	nzbget.append_nzb('6', nzb_content)
	nzbget.append_nzb('7', nzb_content)
	nzbget.append_nzb('8', nzb_content)
	nzbget.append_nzb('9', nzb_content)

def check_list(req):
	queue = nzbget.api.listgroups()
	assert len(req) == len(queue)
	for i in range(len(req)):
		assert req[i] == queue[i]['NZBID']

def test_editqueue_move_offset(nzbget):
	prepare_queue(nzbget)

	check_list([1,2,3,4,5,6,7,8,9])

	nzbget.api.editqueue('GroupMoveOffset', '1', [2])
	check_list([1,3,2,4,5,6,7,8,9])

	nzbget.api.editqueue('GroupMoveOffset', '-1', [2])
	check_list([1,2,3,4,5,6,7,8,9])

	nzbget.api.editqueue('GroupMoveOffset', '2', [4,5,6])
	check_list([1,2,3,7,8,4,5,6,9])

	nzbget.api.editqueue('GroupMoveOffset', '-2', [4,5,6])
	check_list([1,2,3,4,5,6,7,8,9])

	nzbget.api.editqueue('GroupMoveOffset', '5', [4,6])
	check_list([1,2,3,5,7,8,9,4,6])

	nzbget.api.editqueue('GroupMoveOffset', '-5', [2,5,7])
	check_list([2,5,7,1,3,8,9,4,6])

def test_editqueue_move_before_after(nzbget):
	prepare_queue(nzbget)

	check_list([1,2,3,4,5,6,7,8,9])

	nzbget.api.editqueue('GroupMoveBefore', '2', [8,9])
	check_list([1,8,9,2,3,4,5,6,7])

	nzbget.api.editqueue('GroupMoveBefore', '2', [8,9])
	check_list([1,8,9,2,3,4,5,6,7])

	nzbget.api.editqueue('GroupMoveAfter', '2', [8,9])
	check_list([1,2,8,9,3,4,5,6,7])

	nzbget.api.editqueue('GroupMoveAfter', '7', [1,8,9])
	check_list([2,3,4,5,6,7,1,8,9])

	nzbget.api.editqueue('GroupMoveBefore', '2', [5,1,9])
	check_list([5,1,9,2,3,4,6,7,8])

	nzbget.api.editqueue('GroupMoveBefore', '2', [5,1,9])
	check_list([5,1,9,2,3,4,6,7,8])

	nzbget.api.editqueue('GroupMoveAfter', '2', [5,1,9])
	check_list([2,5,1,9,3,4,6,7,8])

	nzbget.api.editqueue('GroupMoveAfter', '8', [5,2])
	check_list([1,9,3,4,6,7,8,2,5])



import pytest
import subprocess
import os
import time
import shutil
import base64
import distutils.spawn
try:
	from xmlrpclib import ServerProxy # python 2
except ImportError:
	from xmlrpc.client import ServerProxy # python 3

nzbget_srcdir = os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
nzbget_maindir = nzbget_srcdir + '/tests/testdata/nzbget.temp'
nzbget_configfile = nzbget_maindir + '/nzbget.conf'
nzbget_rpcurl = 'http://127.0.0.1:6789/xmlrpc';

exe_ext = '.exe' if os.name == 'nt' else ''

nzbget_bin = nzbget_srcdir + '/nzbget' + exe_ext
nserv_datadir = nzbget_srcdir + '/tests/testdata/nserv.temp'

sevenzip_bin = distutils.spawn.find_executable('7z')
if sevenzip_bin is None:
	sevenzip_bin = nzbget_srcdir + '/7z' + exe_ext

par2_bin = distutils.spawn.find_executable('par2')
if par2_bin is None:
	par2_bin = nzbget_srcdir + '/par2' + exe_ext

has_failures = False

def pytest_addoption(parser):
	parser.addini('nzbget_bin', 'path to nzbget binary', default=nzbget_bin)
	parser.addini('nserv_datadir', 'path to nserv datadir', default=nserv_datadir)
	parser.addini('nzbget_maindir', 'path to nzbget maindir', default=nzbget_maindir)
	parser.addini('sevenzip_bin', 'path to 7-zip', default=sevenzip_bin)
	parser.addini('par2_bin', 'path to par2 binary', default=par2_bin)
	parser.addoption("--hold", action="store_true", help="Hold at the end of test (keep NZBGet running)")

def check_config():
	global nzbget_bin
	nzbget_bin = pytest.config.getini('nzbget_bin')
	if not os.path.exists(nzbget_bin):
		pytest.exit('Could not find nzbget binary at ' + nzbget_bin + '. Alternative path can be set via pytest ini option "nzbget_bin".')

	global sevenzip_bin, par2_bin
	sevenzip_bin = pytest.config.getini('sevenzip_bin')
	par2_bin = pytest.config.getini('par2_bin')
	if not os.path.exists(sevenzip_bin):
		pytest.exit('Could not find 7-zip binary in search path or at ' + sevenzip_bin + '. Alternative path can be set via pytest ini option "sevenzip_bin".')
	if not os.path.exists(par2_bin):
		pytest.exit('Could not find par2 binary in search path or at ' + par2_bin + '. Alternative path can be set via pytest ini option "par2_bin".')

	global nserv_datadir
	nserv_datadir = pytest.config.getini('nserv_datadir')

	global nzbget_maindir
	nzbget_maindir = pytest.config.getini('nzbget_maindir')
	global nzbget_configfile
	nzbget_configfile = nzbget_maindir + '/nzbget.conf'

pytest.check_config = check_config

class NServ:

	def __init__(self):
		self.process = subprocess.Popen([nzbget_bin, '--nserv', '-d', nserv_datadir, '-v', '0', '-i', '2'])

	def finalize(self):
		self.process.kill()

@pytest.fixture(scope='session')

def nserv(request):
	check_config()

	instance = NServ()
	request.addfinalizer(instance.finalize)
	return instance


class Nzbget:

	def __init__(self, options, session):
		self.options = options
		self.session = session
		self.api = ServerProxy(nzbget_rpcurl)
		self.prepare_session()
		self.process = subprocess.Popen([nzbget_bin, '-c', nzbget_configfile, '-s', '-o', 'outputmode=log'])
		self.wait_until_started()

	def finalize(self):
		if pytest.config.getoption("--hold"):
			print('\nNZBGet is still running, press Ctrl+C to quit')
			time.sleep(100000)
		self.process.kill()
		if not has_failures:
			self.remove_tempdir()

	def remove_tempdir(self):
		attempt = 1
		completed = False
		while not completed:
			try:
				if os.path.exists(nzbget_maindir + '.old'):
					shutil.rmtree(nzbget_maindir + '.old')
				if os.path.exists(nzbget_maindir):
					os.rename(nzbget_maindir, nzbget_maindir + '.old')
					shutil.rmtree(nzbget_maindir + '.old')
				completed = True
			except Exception:
				if attempt > 20:
					raise
				attempt += 1
				time.sleep(0.2)

	def prepare_session(self):
		self.remove_tempdir()
		os.makedirs(nzbget_maindir)
		config = open(nzbget_configfile, 'w')
		config.write('MainDir=' + nzbget_maindir + '\n')
		config.write('DestDir=${MainDir}/complete\n')
		config.write('InterDir=${MainDir}/intermediate\n')
		config.write('TempDir=${MainDir}/temp\n')
		config.write('QueueDir=${MainDir}/queue\n')
		config.write('NzbDir=${MainDir}/nzb\n')
		config.write('LogFile=${MainDir}/nzbget.log\n')
		config.write('SevenZipCmd=' + sevenzip_bin + '\n')
		config.write('WriteLog=append\n')
		config.write('DetailTarget=log\n')
		config.write('InfoTarget=log\n')
		config.write('WarningTarget=log\n')
		config.write('ErrorTarget=log\n')
		config.write('DebugTarget=none\n')
		config.write('CrashTrace=no\n')
		config.write('CrashDump=yes\n')
		config.write('ContinuePartial=no\n')
		config.write('DirectWrite=yes\n')
		config.write('ArticleCache=500\n')
		config.write('WriteBuffer=1024\n')
		config.write('NzbDirInterval=0\n')
		config.write('FlushQueue=no\n')
		config.write('WebDir=' + nzbget_srcdir + '/webui\n')
		config.write('ConfigTemplate=' + nzbget_srcdir + '/nzbget.conf\n')
		config.write('ControlUsername=\n')
		config.write('ControlPassword=\n')
		config.write('ControlIP=127.0.0.1\n')
		config.write('Server1.host=127.0.0.1\n')
		config.write('Server1.port=6791\n')
		config.write('Server1.connections=10\n')
		config.write('Server1.level=0\n')
		config.write('Server2.host=127.0.0.1\n')
		config.write('Server2.port=6792\n')
		config.write('Server2.connections=10\n')
		config.write('Server2.level=1\n')
		config.write('Server2.active=no\n')
		for opt in self.options:
			config.write(opt + '\n')

	def wait_until_started(self):
		print('Waiting for nzbget to start')
		stat = None
		for x in range(0, 3):
			try:
				stat = self.api.status()
			except Exception:
				time.sleep(0.5)

		if stat is None:
			raise Exception('Could not start nzbget')
		print('Started')

	def append_nzb(self, nzb_name, nzb_content, unpack = None, dupekey = '', dupescore = 0, dupemode = 'FORCE', params = None):
		nzbcontent64 = base64.standard_b64encode(nzb_content)
		if params is None:
			params = []
		if unpack == True:
			params.append(('*unpack:', 'yes'))
		elif unpack == False:
			params.append(('*unpack:', 'no'))
		return self.api.append(nzb_name, nzbcontent64, 'test', 0, False, False, dupekey, dupescore, dupemode, params)

	def load_nzb(self, nzb_name):
		fullfilename = nserv_datadir + '/' + nzb_name
		in_file = open(fullfilename, 'r')

		nzbcontent = in_file.read()

		in_file.close()
		return nzbcontent

	def download_nzb(self, nzb_name, nzb_content = None, unpack = None, dupekey = '', dupescore = 0, dupemode = 'FORCE', params = None):
		if not nzb_content:
			nzb_content = self.load_nzb(nzb_name)
		self.append_nzb(nzb_name, nzb_content, unpack, dupekey, dupescore, dupemode, params)
		hist = self.wait_nzb(nzb_name)
		return hist

	def wait_nzb(self, nzb_name):
		print('Waiting for download completion')
		hist = None
		while not hist:
			history = self.api.history()
			for hist1 in history:
				if hist1['NZBFilename'] == nzb_name:
					hist = hist1
					break
			time.sleep(0.1)
		return hist

	def clear(self):
		self.api.editqueue('HistoryFinalDelete', 0, '', range(1, 1000));

@pytest.fixture(scope='module')

def nzbget(request):
	check_config()

	instance = Nzbget(getattr(request.module, 'nzbget_options', []), request.session)
	request.addfinalizer(instance.finalize)
	return instance


@pytest.hookimpl(tryfirst=True, hookwrapper=True)

def pytest_runtest_makereport(item, call):

	# execute all other hooks to obtain the report object

	outcome = yield

	rep = outcome.get_result()
	global has_failures

	has_failures = has_failures or rep.failed


#!/usr/bin/env python
#
# E-Mail post-processing script for NZBGet
#
# Copyright (C) 2013-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


##############################################################################
### NZBGET POST-PROCESSING SCRIPT                                          ###

# Send E-Mail notification.
#
# This script sends E-Mail notification when the job is done.
#
# NOTE: This script requires Python to be installed on your system.

##############################################################################
### OPTIONS                                                       ###

# When to send the message (Always, OnFailure).
#SendMail=Always

# Email address you want this email to be sent from.
#From="NZBGet" <myaccount@gmail.com>

# Email address you want this email to be sent to.
#
# Multiple addresses can be separated with comma.
#To=myaccount@gmail.com

# SMTP server host.
#Server=smtp.gmail.com

# SMTP server port (1-65535).
#Port=25

# Secure communication using TLS/SSL (yes, no, force).
#  no    - plain text communication (insecure);
#  yes   - switch to secure session using StartTLS command;
#  force - start secure session on encrypted socket.
#Encryption=yes

# SMTP server user name, if required.
#Username=myaccount

# SMTP server password, if required.
#Password=mypass

# To check connection parameters click the button.
#ConnectionTest@Send Test E-Mail

# Append statistics to the message (yes, no).
#Statistics=yes

# Append list of files to the message (yes, no).
#
# Add the list of downloaded files (the content of destination directory).
#FileList=yes

# Append broken-log to the message (yes, no).
#
# Add the content of file _brokenlog.txt. This file contains the list of damaged
# files and the result of par-check/repair. For successful downloads the broken-log
# is usually deleted by cleanup-script and therefore is not sent.
#BrokenLog=yes

# Append nzb log to the message (Always, Never, OnFailure).
#
# Add the download and post-processing log of active job.
#NzbLog=OnFailure

### NZBGET POST-PROCESSING SCRIPT                                          ###
##############################################################################


import os
import sys
import datetime
import smtplib

from email.mime.text import MIMEText
from email.header import Header
try: # python 2
	from urllib2 import quote
	from xmlrpclib import ServerProxy
except ImportError: # python 3
	from urllib.parse import quote
	from xmlrpc.client import ServerProxy

# Exit codes used by NZBGet
POSTPROCESS_SUCCESS=93
POSTPROCESS_ERROR=94
POSTPROCESS_NONE=95

# Check if the script is called from nzbget 15.0 or later
if not 'NZBOP_NZBLOG' in os.environ:
	print('*** NZBGet post-processing script ***')
	print('This script is supposed to be called from nzbget (15.0 or later).')
	sys.exit(POSTPROCESS_ERROR)

print('[DETAIL] Script successfully started')
sys.stdout.flush()

required_options = ('NZBPO_FROM', 'NZBPO_TO', 'NZBPO_SERVER', 'NZBPO_PORT', 'NZBPO_ENCRYPTION', 'NZBPO_USERNAME', 'NZBPO_PASSWORD')
for	optname in required_options:
	if (not optname in os.environ):
		print('[ERROR] Option %s is missing in configuration file. Please check script settings' % optname[6:])
		sys.exit(POSTPROCESS_ERROR)

# Check if the script is executed from settings page with a custom command
command = os.environ.get('NZBCP_COMMAND')
test_mode = command == 'ConnectionTest'
if command != None and not test_mode:
	print('[ERROR] Invalid command ' + command)
	sys.exit(POSTPROCESS_ERROR)

status = os.environ.get('NZBPP_STATUS') if not test_mode else 'SUCCESS/ALL'
total_status = os.environ.get('NZBPP_TOTALSTATUS') if not test_mode else 'SUCCESS'

# If any script fails the status of the item in the history is "WARNING/SCRIPT".
# This status however is not passed to pp-scripts in the env var "NZBPP_STATUS"
# because most scripts are independent of each other and should work even
# if a previous script has failed. But not in the case of E-Mail script,
# which should take the status of the previous scripts into account as well.
if total_status == 'SUCCESS' and os.environ.get('NZBPP_SCRIPTSTATUS') == 'FAILURE':
	total_status = 'WARNING'
	status = 'WARNING/SCRIPT'

success = total_status == 'SUCCESS'

if success and os.environ.get('NZBPO_SENDMAIL') == 'OnFailure' and not test_mode:
	print('[INFO] Skipping sending of message for successful download')
	sys.exit(POSTPROCESS_NONE)

if success:
	subject = 'Success for "%s"' % (os.environ.get('NZBPP_NZBNAME', 'Test download'))
	text = 'Download of "%s" has successfully completed.' % (os.environ.get('NZBPP_NZBNAME', 'Test download'))
else:
	subject = 'Failure for "%s"' % (os.environ['NZBPP_NZBNAME'])
	text = 'Download of "%s" has failed.' % (os.environ['NZBPP_NZBNAME'])

text += '\nStatus: %s' % status

if (os.environ.get('NZBPO_STATISTICS') == 'yes' or \
	os.environ.get('NZBPO_NZBLOG') == 'Always' or \
	(os.environ.get('NZBPO_NZBLOG') == 'OnFailure' and not success)) and \
	not test_mode:
	# To get statistics or the post-processing log we connect to NZBGet via XML-RPC.
	# For more info visit http://nzbget.net/api
	# First we need to know connection info: host, port and password of NZBGet server.
	# NZBGet passes all configuration options to post-processing script as
	# environment variables.
	host = os.environ['NZBOP_CONTROLIP'];
	port = os.environ['NZBOP_CONTROLPORT'];
	username = os.environ['NZBOP_CONTROLUSERNAME'];
	password = os.environ['NZBOP_CONTROLPASSWORD'];

	if host == '0.0.0.0': host = '127.0.0.1'

	# Build a URL for XML-RPC requests
	rpcUrl = 'http://%s:%s@%s:%s/xmlrpc' % (quote(username), quote(password), host, port);

	# Create remote server object
	server = ServerProxy(rpcUrl)

if os.environ.get('NZBPO_STATISTICS') == 'yes' and not test_mode:
	# Find correct nzb in method listgroups
	groups = server.listgroups(0)
	nzbID = int(os.environ['NZBPP_NZBID'])
	for nzbGroup in groups:
		if nzbGroup['NZBID'] == nzbID:
			break

	text += '\n\nStatistics:';

	# add download size
	DownloadedSize = float(nzbGroup['DownloadedSizeMB'])
	unit = ' MB'
	if DownloadedSize > 1024:
		DownloadedSize = DownloadedSize / 1024 # GB
		unit = ' GB'
	text += '\nDownloaded size: %.2f' % (DownloadedSize) + unit

	# add average download speed
	DownloadedSizeMB = float(nzbGroup['DownloadedSizeMB'])
	DownloadTimeSec = float(nzbGroup['DownloadTimeSec'])
	if DownloadTimeSec > 0: # check x/0 errors
		avespeed = (DownloadedSizeMB/DownloadTimeSec) # MB/s
		unit = ' MB/s'
		if avespeed < 1:
			avespeed = avespeed * 1024 # KB/s
			unit = ' KB/s'
		text += '\nAverage download speed: %.2f' % (avespeed) + unit

	def format_time_sec(sec):
		Hour = sec/3600
		Min = (sec - (sec/3600)*3600)/60
		Sec = (sec - (sec/3600)*3600)%60
		return '%d:%02d:%02d' % (Hour,Min,Sec)

	# add times
	text += '\nTotal time: ' + format_time_sec(int(nzbGroup['DownloadTimeSec']) + int(nzbGroup['PostTotalTimeSec']))
	text += '\nDownload time: ' + format_time_sec(int(nzbGroup['DownloadTimeSec']))
	text += '\nVerification time: ' + format_time_sec(int(nzbGroup['ParTimeSec']) - int(nzbGroup['RepairTimeSec']))
	text += '\nRepair time: ' + format_time_sec(int(nzbGroup['RepairTimeSec']))
	text += '\nUnpack time: ' + format_time_sec(int(nzbGroup['UnpackTimeSec']))

# add list of downloaded files
files = False
if os.environ.get('NZBPO_FILELIST') == 'yes' and not test_mode:
	text += '\n\nFiles:'
	for dirname, dirnames, filenames in os.walk(os.environ['NZBPP_DIRECTORY']):
		for filename in filenames:
			text += '\n' + os.path.join(dirname, filename)[len(os.environ['NZBPP_DIRECTORY']) + 1:]
			files = True
	if not files:
		text += '\n<no files found in the destination directory (moved by a script?)>'

# add _brokenlog.txt (if exists)
if os.environ.get('NZBPO_BROKENLOG') == 'yes' and not test_mode:
	brokenlog = '%s/_brokenlog.txt' % os.environ['NZBPP_DIRECTORY']
	if os.path.exists(brokenlog):
		text += '\n\nBrokenlog:\n' + open(brokenlog, 'r').read().strip()

# add post-processing log
if (os.environ.get('NZBPO_NZBLOG') == 'Always' or \
	(os.environ.get('NZBPO_NZBLOG') == 'OnFailure' and not success)) and \
	not test_mode:

	# To get the item log we connect to NZBGet via XML-RPC and call
	# method "loadlog", which returns the log for a given nzb item.
	# For more info visit http://nzbget.net/api

	# Call remote method 'loadlog'
	nzbid = int(os.environ['NZBPP_NZBID'])
	log = server.loadlog(nzbid, 0, 10000)

	# Now iterate through entries and save them to message text
	if len(log) > 0:
		text += '\n\nNzb-log:';
		for entry in log:
			text += '\n%s\t%s\t%s' % (entry['Kind'], datetime.datetime.fromtimestamp(int(entry['Time'])), entry['Text'])

# Create message
print('[DETAIL] Creating Email')
msg = MIMEText(text.encode('utf-8'), 'plain', 'utf-8')
msg['Subject'] = Header(subject, 'utf-8')
msg['From'] = os.environ['NZBPO_FROM']
msg['To'] = os.environ['NZBPO_TO']
msg['Date'] = datetime.datetime.utcnow().strftime("%a, %d %b %Y %H:%M:%S +0000")
msg['X-Application'] = 'NZBGet'

# Send message
print('[DETAIL] Sending E-Mail')
sys.stdout.flush()
try:
	if os.environ['NZBPO_ENCRYPTION'] == 'force':
		smtp = smtplib.SMTP_SSL(os.environ['NZBPO_SERVER'], os.environ['NZBPO_PORT'])
	else:
		smtp = smtplib.SMTP(os.environ['NZBPO_SERVER'], os.environ['NZBPO_PORT'])

	if os.environ['NZBPO_ENCRYPTION'] == 'yes':
		smtp.starttls()

	if os.environ['NZBPO_USERNAME'] != '' and os.environ['NZBPO_PASSWORD'] != '':
		smtp.login(os.environ['NZBPO_USERNAME'], os.environ['NZBPO_PASSWORD'])

	smtp.sendmail(os.environ['NZBPO_FROM'], os.environ['NZBPO_TO'].split(','), msg.as_string())

	smtp.quit()
except Exception as err:
	print('[ERROR] %s' % err)
	sys.exit(POSTPROCESS_ERROR)

# All OK, returning exit status 'POSTPROCESS_SUCCESS' (int <93>) to let NZBGet know
# that our script has successfully completed.
sys.exit(POSTPROCESS_SUCCESS)



import os
import subprocess
import pytest


@pytest.fixture(scope='session', autouse=True)
def prepare_testdata(request):
	print('Preparing test data for "unpack"')

	nserv_datadir = pytest.config.getini('nserv_datadir')
	nzbget_bin = pytest.config.getini('nzbget_bin')
	sevenzip_bin = pytest.config.getini('sevenzip_bin')
	par2_bin = pytest.config.getini('par2_bin')

	if not os.path.exists(nserv_datadir):
		print('Creating nserv datadir')
		os.makedirs(nserv_datadir)

	if not os.path.exists(nserv_datadir + '/unpack-damaged.nzb'):
		create_test_file(nserv_datadir + '/unpack-damaged', sevenzip_bin, 3, 1)
		os.chdir(nserv_datadir + '/unpack-damaged')

		if 0 != subprocess.call([par2_bin, 'c', '-b100', 'unpackcrc-damaged.par2', '*']):
			pytest.exit('Test file generation failed')

		outf = open("3mb.7z.001","rb+")
		outf.seek(100000)
		outf.write(b"\x0a\x1b\x2c")
		outf.close()

	if not os.path.exists(nserv_datadir + '/unpackcrc-par.nzb'):
		create_test_file(nserv_datadir + '/unpackcrc-par', sevenzip_bin, 3, 1)
		os.chdir(nserv_datadir + '/unpackcrc-par')

		outf = open("3mb.7z.001","rb+")
		outf.seek(100000)
		outf.write(b"\x0a\x1b\x2c")
		outf.close()

		if 0 != subprocess.call([par2_bin, 'c', '-b100', 'unpackcrc-par.par2', '*']):
			pytest.exit('Test file generation failed')

	if not os.path.exists(nserv_datadir + '/unpackcrc-nopar.nzb'):
		create_test_file(nserv_datadir + '/unpackcrc-nopar', sevenzip_bin, 3, 1)
		os.chdir(nserv_datadir + '/unpackcrc-nopar')

		outf = open("3mb.7z.001","rb+")
		outf.seek(100000)
		outf.write(b"\x0a\x1b\x2c")
		outf.close()

	if 0 != subprocess.call([nzbget_bin, '--nserv', '-d', nserv_datadir, '-v', '2', '-z', '3000', '-q']):
		pytest.exit('Test file generation failed')


def create_test_file(bigdir, sevenzip_bin, sizemb, partmb):
	print('Preparing test file (' + str(sizemb) + 'MB)')

	if not os.path.exists(bigdir):
		os.makedirs(bigdir)

	f = open(bigdir + '/' + str(sizemb) + 'mb.dat', 'wb')
	for n in xrange(sizemb / partmb):
		print('Writing block %i from %i' % (n + 1, sizemb / partmb))
		f.write(os.urandom(partmb * 1024 * 1024))
	f.close()

	if 0 != subprocess.call([sevenzip_bin, 'a', bigdir + '/' + str(sizemb) + 'mb.7z', '-mx=0', '-v' + str(partmb) + 'm', bigdir + '/' + str(sizemb) + 'mb.dat']):
		pytest.exit('Test file generation failed')

	os.remove(bigdir + '/' + str(sizemb) + 'mb.dat')


def test_unpack_repair(nserv, nzbget):
	hist = nzbget.download_nzb('unpack-damaged.nzb', unpack = True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_unpack_crcerror_par(nserv, nzbget):
	hist = nzbget.download_nzb('unpackcrc-par.nzb', unpack = True)
	assert hist['Status'] == 'FAILURE/UNPACK'

def test_unpack_crcerror_nopar(nserv, nzbget):
	hist = nzbget.download_nzb('unpackcrc-nopar.nzb', unpack = True)
	assert hist['Status'] == 'FAILURE/UNPACK'



nzbget_options = ['ParCheck=auto', 'ParQuick=yes', 'PostStrategy=balanced']

def test_parchecker_healthy(nserv, nzbget):
	hist = nzbget.download_nzb('parchecker2.nzb')
	assert hist['Status'] == 'SUCCESS/HEALTH'

def test_parchecker_repair(nserv, nzbget):
	nzb_content = nzbget.load_nzb('parchecker2.nzb')
	nzb_content = nzb_content.replace('parchecker2/testfile.7z.001?16=45000:3000', 'parchecker2/testfile.7z.001?16=45000:3000!0')
	hist = nzbget.download_nzb('parchecker.repair.nzb', nzb_content)
	assert hist['Status'] == 'SUCCESS/PAR'



nzbget_options = ['ArticleCache=0', 'DirectWrite=no']

def test_small(nserv, nzbget):
	hist = nzbget.download_nzb('small.nzb')
	assert hist['Status'] == 'SUCCESS/HEALTH'

def test_medium_unpack(nserv, nzbget):
	nzb_content = nzbget.load_nzb('medium.nzb')
	hist = nzbget.download_nzb('medium_unpack.nzb', nzb_content, unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'




import os
import shutil
import subprocess
import pytest

@pytest.fixture(scope='session', autouse=True)
def prepare_testdata(request):
	print('Preparing test data for "rename"')
	pytest.check_config()

	nserv_datadir = pytest.config.getini('nserv_datadir')
	nzbget_bin = pytest.config.getini('nzbget_bin')
	sevenzip_bin = pytest.config.getini('sevenzip_bin')
	par2_bin = pytest.config.getini('par2_bin')

	if not os.path.exists(nserv_datadir):
		print('Creating nserv datadir')
		os.makedirs(nserv_datadir)

	nzbget_srcdir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(__file__))))
	testdata_dir = nzbget_srcdir + '/tests/testdata'

	if not os.path.exists(nserv_datadir + '/parrename'):
		os.makedirs(nserv_datadir + '/parrename')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile3.part01.rar', nserv_datadir + '/parrename/testfile3.part01.rar')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile3.part02.rar', nserv_datadir + '/parrename/testfile3.part02.rar')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile3.part03.rar', nserv_datadir + '/parrename/testfile3.part03.rar')
		os.chdir(nserv_datadir + '/parrename')
		if 0 != subprocess.call([par2_bin, 'c', '-b20', 'parrename.par2', '*']):
			pytest.exit('Test file generation failed')
		os.rename(nserv_datadir + '/parrename/testfile3.part01.rar', nserv_datadir + '/parrename/abc.21')
		os.rename(nserv_datadir + '/parrename/testfile3.part02.rar', nserv_datadir + '/parrename/abc.02')
		os.rename(nserv_datadir + '/parrename/testfile3.part03.rar', nserv_datadir + '/parrename/abc.15')

	def prepare_test(dirname, testfile):
		if not os.path.exists(nserv_datadir + '/' + dirname + '.nzb'):
			os.makedirs(nserv_datadir + '/' + dirname)
			shutil.copyfile(testdata_dir + '/rarrenamer/' + testfile + '.part01.rar', nserv_datadir + '/' + dirname + '/abc.21')
			shutil.copyfile(testdata_dir + '/rarrenamer/' + testfile + '.part02.rar', nserv_datadir + '/' + dirname + '/abc.02')
			shutil.copyfile(testdata_dir + '/rarrenamer/' + testfile + '.part03.rar', nserv_datadir + '/' + dirname + '/abc.15')
			os.chdir(nserv_datadir + '/' + dirname)
			if 0 != subprocess.call([par2_bin, 'c', '-b20', 'parrename.par2', '*']):
				pytest.exit('Test file generation failed')

	prepare_test('rarrename3', 'testfile3')
	prepare_test('rarrename5', 'testfile5')
	prepare_test('rarrename3encdata', 'testfile3encdata')
	prepare_test('rarrename5encdata', 'testfile5encdata')
	prepare_test('rarrename3encnam', 'testfile3encnam')
	prepare_test('rarrename5encnam', 'testfile5encnam')

	if not os.path.exists(nserv_datadir + '/rarrename2sets'):
		os.makedirs(nserv_datadir + '/rarrename2sets')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile3.part01.rar', nserv_datadir + '/rarrename2sets/abc.21')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile3.part02.rar', nserv_datadir + '/rarrename2sets/abc.02')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile3.part03.rar', nserv_datadir + '/rarrename2sets/abc.15')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile5.part01.rar', nserv_datadir + '/rarrename2sets/abc.22')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile5.part02.rar', nserv_datadir + '/rarrename2sets/abc.03')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile5.part03.rar', nserv_datadir + '/rarrename2sets/abc.14')

	if not os.path.exists(nserv_datadir + '/rarrename3oldnam'):
		os.makedirs(nserv_datadir + '/rarrename3oldnam')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile3oldnam.rar', nserv_datadir + '/rarrename3oldnam/abc.61')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile3oldnam.r00', nserv_datadir + '/rarrename3oldnam/abc.32')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile3oldnam.r01', nserv_datadir + '/rarrename3oldnam/abc.45')

	if not os.path.exists(nserv_datadir + '/rarrename3badext'):
		os.makedirs(nserv_datadir + '/rarrename3badext')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile3oldnam.rar', nserv_datadir + '/rarrename3badext/testfile3oldnam.rar')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile3oldnam.r00', nserv_datadir + '/rarrename3badext/testfile3oldnam.r03')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile3oldnam.r01', nserv_datadir + '/rarrename3badext/testfile3oldnam.r02')

	if not os.path.exists(nserv_datadir + '/rarrename5badext'):
		os.makedirs(nserv_datadir + '/rarrename5badext')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile3.part01.rar', nserv_datadir + '/rarrename5badext/testfile3.part01.rar')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile3.part02.rar', nserv_datadir + '/rarrename5badext/testfile3.part0002.rar')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile3.part03.rar', nserv_datadir + '/rarrename5badext/testfile3.part03.rar')

	if not os.path.exists(nserv_datadir + '/rar3ignoreext'):
		os.makedirs(nserv_datadir + '/rar3ignoreext')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile3.part01.rar', nserv_datadir + '/rar3ignoreext/testfile3-1.cbr')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile3.part02.rar', nserv_datadir + '/rar3ignoreext/testfile3-2.cbr')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile3.part03.rar', nserv_datadir + '/rar3ignoreext/testfile3-3.cbr')

	if not os.path.exists(nserv_datadir + '/parjoin1.nzb'):
		os.makedirs(nserv_datadir + '/parjoin1')
		shutil.copyfile(testdata_dir + '/parchecker/testfile.dat', nserv_datadir + '/parjoin1/testfile.dat')
		shutil.copyfile(testdata_dir + '/parchecker/testfile.nfo', nserv_datadir + '/parjoin1/testfile.nfo')
		shutil.copyfile(testdata_dir + '/parchecker/testfile.par2', nserv_datadir + '/parjoin1/testfile.par2')
		shutil.copyfile(testdata_dir + '/parchecker/testfile.vol00+1.PAR2', nserv_datadir + '/parjoin1/testfile.vol00+1.PAR2')
		split_test_file(nserv_datadir + '/parjoin1', 'testfile.dat', 50244);

	if not os.path.exists(nserv_datadir + '/parjoin2.nzb'):
		os.makedirs(nserv_datadir + '/parjoin2')
		shutil.copyfile(testdata_dir + '/parchecker/testfile.dat', nserv_datadir + '/parjoin2/testfile.dat')
		shutil.copyfile(testdata_dir + '/parchecker/testfile.nfo', nserv_datadir + '/parjoin2/testfile.nfo')
		shutil.copyfile(testdata_dir + '/parchecker/testfile.par2', nserv_datadir + '/parjoin2/testfile.par2')
		shutil.copyfile(testdata_dir + '/parchecker/testfile.vol00+1.PAR2', nserv_datadir + '/parjoin2/testfile.vol00+1.PAR2')
		split_test_file(nserv_datadir + '/parjoin2', 'testfile.dat', 50244);
		os.rename(nserv_datadir + '/parjoin2/testfile.dat.001', nserv_datadir + '/parjoin2/renamed.001')
		os.rename(nserv_datadir + '/parjoin2/testfile.dat.002', nserv_datadir + '/parjoin2/renamed.002')
		os.rename(nserv_datadir + '/parjoin2/testfile.dat.003', nserv_datadir + '/parjoin2/renamed.003')

	if 0 != subprocess.call([nzbget_bin, '--nserv', '-d', nserv_datadir, '-v', '2', '-z', '3000', '-q']):
		pytest.exit('Test file generation failed')

	if not os.path.exists(nserv_datadir + '/parjoin3.nzb'):
		os.makedirs(nserv_datadir + '/parjoin3')
		create_test_file(nserv_datadir + '/parjoin3', None, 20, 1)
		os.chdir(nserv_datadir + '/parjoin3')
		if 0 != subprocess.call([par2_bin, 'c', '-b100', 'parrename.par2', '*']):
			pytest.exit('Test file generation failed')
		split_test_file(nserv_datadir + '/parjoin3', '20mb.dat', 7*1024*1024);
		os.rename(nserv_datadir + '/parjoin3/20mb.dat.001', nserv_datadir + '/parjoin3/renamed.001')
		os.rename(nserv_datadir + '/parjoin3/20mb.dat.002', nserv_datadir + '/parjoin3/renamed.002')
		os.rename(nserv_datadir + '/parjoin3/20mb.dat.003', nserv_datadir + '/parjoin3/renamed.003')
		if 0 != subprocess.call([nzbget_bin, '--nserv', '-d', nserv_datadir, '-v', '2', '-z', '100000', '-q']):
			pytest.exit('Test file generation failed')

	if not os.path.exists(nserv_datadir + '/rarrename3sm'):
		os.makedirs(nserv_datadir + '/rarrename3sm')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile3oldnam.rar', nserv_datadir + '/rarrename3sm/abc.61')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile3oldnam.r00', nserv_datadir + '/rarrename3sm/abc.32')
		shutil.copyfile(testdata_dir + '/rarrenamer/testfile3oldnam.r01', nserv_datadir + '/rarrename3sm/abc.45')
		os.chdir(nserv_datadir + '/rarrename3sm')
		if 0 != subprocess.call([par2_bin, 'c', '-b100', 'parrename.par2', '*']):
			pytest.exit('Test file generation failed')
		if 0 != subprocess.call([nzbget_bin, '--nserv', '-d', nserv_datadir, '-v', '2', '-z', '500', '-q']):
			pytest.exit('Test file generation failed')

	if not os.path.exists(nserv_datadir + '/obfuscated1.nzb'):
		create_test_file(nserv_datadir + '/obfuscated1', sevenzip_bin, 5, 1)
		os.chdir(nserv_datadir + '/obfuscated1')
		if 0 != subprocess.call([par2_bin, 'c', '-b100', 'parrename.par2', '*']):
			pytest.exit('Test file generation failed')
		os.rename(nserv_datadir + '/obfuscated1/5mb.7z.001', nserv_datadir + '/obfuscated1/abc.51')
		os.rename(nserv_datadir + '/obfuscated1/5mb.7z.002', nserv_datadir + '/obfuscated1/abc.01')
		os.rename(nserv_datadir + '/obfuscated1/5mb.7z.003', nserv_datadir + '/obfuscated1/abc.21')
		os.rename(nserv_datadir + '/obfuscated1/5mb.7z.004', nserv_datadir + '/obfuscated1/abc.34')
		os.rename(nserv_datadir + '/obfuscated1/5mb.7z.005', nserv_datadir + '/obfuscated1/abc.17')
		os.rename(nserv_datadir + '/obfuscated1/5mb.7z.006', nserv_datadir + '/obfuscated1/abc.00')
		if 0 != subprocess.call([nzbget_bin, '--nserv', '-d', nserv_datadir, '-v', '2', '-z', '100000', '-q']):
			pytest.exit('Test file generation failed')

	if not os.path.exists(nserv_datadir + '/obfuscated2.nzb'):
		create_test_file(nserv_datadir + '/obfuscated2', sevenzip_bin, 5, 1)
		os.chdir(nserv_datadir + '/obfuscated2')
		if 0 != subprocess.call([par2_bin, 'c', '-b100', 'parrename.par2', '*']):
			pytest.exit('Test file generation failed')
		os.rename(nserv_datadir + '/obfuscated2/5mb.7z.001', nserv_datadir + '/obfuscated2/abc.51')
		os.rename(nserv_datadir + '/obfuscated2/5mb.7z.002', nserv_datadir + '/obfuscated2/abc.01')
		os.rename(nserv_datadir + '/obfuscated2/5mb.7z.003', nserv_datadir + '/obfuscated2/abc.21')
		os.rename(nserv_datadir + '/obfuscated2/5mb.7z.004', nserv_datadir + '/obfuscated2/abc.34')
		os.rename(nserv_datadir + '/obfuscated2/5mb.7z.005', nserv_datadir + '/obfuscated2/abc.17')
		os.rename(nserv_datadir + '/obfuscated2/5mb.7z.006', nserv_datadir + '/obfuscated2/abc.00')
		os.rename(nserv_datadir + '/obfuscated2/parrename.par2', nserv_datadir + '/obfuscated2/abc.90')
		os.rename(nserv_datadir + '/obfuscated2/parrename.vol0+1.par2', nserv_datadir + '/obfuscated2/abc.95')
		os.rename(nserv_datadir + '/obfuscated2/parrename.vol1+2.par2', nserv_datadir + '/obfuscated2/abc.91')
		os.rename(nserv_datadir + '/obfuscated2/parrename.vol3+2.par2', nserv_datadir + '/obfuscated2/abc.92')
		if 0 != subprocess.call([nzbget_bin, '--nserv', '-d', nserv_datadir, '-v', '2', '-z', '100000', '-q']):
			pytest.exit('Test file generation failed')

	if not os.path.exists(nserv_datadir + '/obfuscated3.nzb'):
		create_test_file(nserv_datadir + '/obfuscated3', sevenzip_bin, 100, 10)
		os.chdir(nserv_datadir + '/obfuscated3')
		if 0 != subprocess.call([par2_bin, 'c', '-b100', 'parrename.par2', '*']):
			pytest.exit('Test file generation failed')
		os.rename(nserv_datadir + '/obfuscated3/100mb.7z.001', nserv_datadir + '/obfuscated3/abc.51')
		os.rename(nserv_datadir + '/obfuscated3/100mb.7z.002', nserv_datadir + '/obfuscated3/abc.01')
		os.rename(nserv_datadir + '/obfuscated3/100mb.7z.003', nserv_datadir + '/obfuscated3/abc.21')
		os.rename(nserv_datadir + '/obfuscated3/100mb.7z.004', nserv_datadir + '/obfuscated3/abc.34')
		os.rename(nserv_datadir + '/obfuscated3/100mb.7z.005', nserv_datadir + '/obfuscated3/abc.17')
		os.rename(nserv_datadir + '/obfuscated3/100mb.7z.006', nserv_datadir + '/obfuscated3/abc.60')
		os.rename(nserv_datadir + '/obfuscated3/100mb.7z.007', nserv_datadir + '/obfuscated3/abc.32')
		os.rename(nserv_datadir + '/obfuscated3/100mb.7z.008', nserv_datadir + '/obfuscated3/abc.35')
		os.rename(nserv_datadir + '/obfuscated3/100mb.7z.009', nserv_datadir + '/obfuscated3/abc.41')
		os.rename(nserv_datadir + '/obfuscated3/100mb.7z.010', nserv_datadir + '/obfuscated3/abc.50')
		os.rename(nserv_datadir + '/obfuscated3/100mb.7z.011', nserv_datadir + '/obfuscated3/abc.43')
		os.rename(nserv_datadir + '/obfuscated3/parrename.par2', nserv_datadir + '/obfuscated3/abc.00')
		os.rename(nserv_datadir + '/obfuscated3/parrename.vol0+1.par2', nserv_datadir + '/obfuscated3/abc.02')
		os.rename(nserv_datadir + '/obfuscated3/parrename.vol1+2.par2', nserv_datadir + '/obfuscated3/abc.91')
		os.rename(nserv_datadir + '/obfuscated3/parrename.vol3+2.par2', nserv_datadir + '/obfuscated3/abc.92')
		if 0 != subprocess.call([nzbget_bin, '--nserv', '-d', nserv_datadir, '-v', '2', '-z', '100000', '-q']):
			pytest.exit('Test file generation failed')

def create_test_file(bigdir, sevenzip_bin, sizemb, partmb):
	print('Preparing test file (' + str(sizemb) + 'MB)')

	if not os.path.exists(bigdir):
		os.makedirs(bigdir)

	f = open(bigdir + '/' + str(sizemb) + 'mb.dat', 'wb')
	for n in xrange(sizemb / partmb):
		print('Writing block %i from %i' % (n + 1, sizemb / partmb))
		f.write(os.urandom(partmb * 1024 * 1024))
	f.close()

	if sevenzip_bin != None:
		if 0 != subprocess.call([sevenzip_bin, 'a', bigdir + '/' + str(sizemb) + 'mb.7z', '-mx=0', '-v' + str(partmb) + 'm', bigdir + '/' + str(sizemb) + 'mb.dat']):
			pytest.exit('Test file generation failed')

		os.remove(bigdir + '/' + str(sizemb) + 'mb.dat')

def split_test_file(filedir, filename, partsize):
	print('Splitting test file ' + filename)

	inp = open(filedir + '/' + filename, 'rb')
	inp.seek(0,2) # move the cursor to the end of the file
	size = inp.tell()	
	inp.seek(0,0) # move the cursor to the start of the file

	written = 0
	part = 1
	while (written < size):
		block = inp.read(partsize)
		written += partsize
		f = open(filedir + '/' + filename + '.' + str(part).zfill(3), 'wb')
		f.write(block)
		f.close()
		part += 1
	inp.close()

	os.remove(filedir + '/' + filename)



import os

nzbget_options = ['HealthCheck=none', 'ArticleCache=500', 'DirectWrite=yes', 'FileNaming=auto']

def test_small_obfuscated(nserv, nzbget):
	hist = nzbget.download_nzb('small-obfuscated.nzb')
	assert hist['Status'] == 'SUCCESS/HEALTH'
	assert os.path.exists(hist['DestDir'] + '/fsdkhKHGuwuMNBKskd')

def test_small_obfuscated_bad(nserv, nzbget):
	nzb_content = nzbget.load_nzb('small-obfuscated.nzb')
	nzb_content = nzb_content.replace(';fsdkhKHGuwuMNBKskd', ';gpl.txt')
	hist = nzbget.download_nzb('small-obfuscated.mod.nzb', nzb_content, params=[('*naming', 'nzb')])
	assert hist['Status'] == 'SUCCESS/HEALTH'
	assert not os.path.exists(hist['DestDir'] + '/fsdkhKHGuwuMNBKskd')
	assert os.path.exists(hist['DestDir'] + '/gpl.txt')

def test_small(nserv, nzbget):
	hist = nzbget.download_nzb('small.nzb')
	assert hist['Status'] == 'SUCCESS/HEALTH'
	assert os.path.exists(hist['DestDir'] + '/small.dat')

def test_small_changed(nserv, nzbget):
	nzb_content = nzbget.load_nzb('small.nzb')
	nzb_content = nzb_content.replace(';small.dat', ';small-changed.dat')
	hist = nzbget.download_nzb('small-changed.nzb', nzb_content)
	assert hist['Status'] == 'SUCCESS/HEALTH'
	assert os.path.exists(hist['DestDir'] + '/small.dat')



nzbget_options = ['HealthCheck=none', 'ArticleCache=500', 'DirectWrite=yes']

def test_small(nserv, nzbget):
	hist = nzbget.download_nzb('small.nzb')
	assert hist['Status'] == 'SUCCESS/HEALTH'

def test_small_bad(nserv, nzbget):
	nzb_content = nzbget.load_nzb('small.nzb')
	nzb_content = nzb_content.replace('?3=6000:3000', '?3=6000:3000!0')
	hist = nzbget.download_nzb('small.bad.nzb', nzb_content)
	assert hist['Status'] == 'FAILURE/HEALTH'

def test_medium(nserv, nzbget):
	hist = nzbget.download_nzb('medium.nzb')
	assert hist['Status'] == 'SUCCESS/HEALTH'

def test_medium_unpack(nserv, nzbget):
	nzb_content = nzbget.load_nzb('medium.nzb')
	hist = nzbget.download_nzb('medium_unpack.nzb', nzb_content, unpack=True)
	assert hist['Status'] == 'SUCCESS/UNPACK'

def test_large(nserv, nzbget):
	hist = nzbget.download_nzb('large.nzb')
	assert hist['Status'] == 'SUCCESS/HEALTH'

def test_oneserver_small_success(nserv, nzbget):
	nzb_content = nzbget.load_nzb('small.nzb')
	nzb_content = nzb_content.replace('?3=6000:3000', '?3=6000:3000!1')
	hist = nzbget.download_nzb('small.success.nzb', nzb_content)
	assert hist['Status'] == 'SUCCESS/HEALTH'

def test_oneserver_small_bad(nserv, nzbget):
	nzb_content = nzbget.load_nzb('small.nzb')
	nzb_content = nzb_content.replace('?3=6000:3000', '?3=6000:3000!2')
	hist = nzbget.download_nzb('small.bad2.nzb', nzb_content)
	assert hist['Status'] == 'FAILURE/HEALTH'

def test_twoservers_small_success(nserv, nzbget):
	nzbget.api.editserver(2, True)
	nzb_content = nzbget.load_nzb('small.nzb')
	nzb_content = nzb_content.replace('?3=6000:3000', '?3=6000:3000!2')
	hist = nzbget.download_nzb('small.twoservers.nzb', nzb_content)
	nzbget.api.editserver(2, False)
	assert hist['Status'] == 'SUCCESS/HEALTH'

def test_oneserver_medium_failed(nserv, nzbget):
	nzb_content = nzbget.load_nzb('medium.nzb')
	nzb_content = nzb_content.replace(':500000', ':500000!2')
	hist = nzbget.download_nzb('medium-health-failed.nzb', nzb_content)
	assert hist['Status'] == 'FAILURE/HEALTH'

def test_twoservers_medium_success(nserv, nzbget):
	nzbget.api.editserver(2, True)
	nzb_content = nzbget.load_nzb('medium.nzb')
	nzb_content = nzb_content.replace(':500000', ':500000!2')
	hist = nzbget.download_nzb('medium-health-two.nzb', nzb_content)
	nzbget.api.editserver(2, False)
	assert hist['Status'] == 'SUCCESS/HEALTH'
